#include "system.h"
#include "I2C_Driver.h"


//------------------------------------------------------------------------------
static inline void Delay_Mks_1()
{
//  __NOP();
//  __NOP();
//  __NOP();
//  __NOP();
}
//------------------------------------------------------------------------------

#define I2C_START_1_TIME {Delay_Mks_1();} //  START 1
#define I2C_START_2_TIME {Delay_Mks_1();} //  START 2
#define I2C_STOP_TIME {Delay_Mks_1();} //  STOP condition
#define I2C_CLK_HIGH_TIME {} //  
#define I2C_CLK_LOW_TIME_1 {}
#define I2C_CLK_LOW_TIME_2 {}

bool I2C_Error; //  
//------------------------------------------------------------------------------
inline void SCL(bool On)
{
  if(On) P2OUT |= IND_SCL_P2;
  else P2OUT &= ~IND_SCL_P2;
}
//------------------------------------------------------------------------------
inline bool GetSCL()
{
  if(P2OUT & IND_SCL_P2) return true;
  else return false;
}
//------------------------------------------------------------------------------
inline void SDA(bool On)
{
  if(On) P2DIR &= ~IND_SDA_P2;
  else P2DIR |= IND_SDA_P2;
}
//------------------------------------------------------------------------------
inline bool GetSDA()
{
  return P2IN & IND_SDA_P2;
}
//------------------------------------------------------------------------------
void I2C_Init()
{
  int i;

  SDA(1); P2OUT &= ~IND_SDA_P2;
  I2C_Error = false;
  if(!I2C_Start() || !I2C_Stop())
	{
    SCL(0); I2C_CLK_LOW_TIME_1; SDA(1);
    for(i=0;i < 9;i++)
		{
      SCL(0); I2C_CLK_LOW_TIME_1; I2C_CLK_LOW_TIME_1;
      SCL(1); I2C_CLK_HIGH_TIME;
      if(GetSDA())
			{
        I2C_Error=false; I2C_Stop();
        return;
			}
      I2C_CLK_HIGH_TIME;
		}
	}
}
//-------------------------------------------------------------------------------------------------------------------------------
bool I2C_Start()
{
  if(GetSCL() == 0)
	{
    SDA(1); I2C_CLK_LOW_TIME_2;
    SCL(1); I2C_START_1_TIME;
    if(GetSDA() == 0) goto labErr;
    SDA(0);
	}
  else
	{
    I2C_START_1_TIME;
    if(GetSDA() == 0) goto labErr;
    SDA(0);
	}
  I2C_START_2_TIME; SCL(0);
  I2C_CLK_LOW_TIME_1;
  SDA(1);
  return true;
labErr:
  SDA(1); I2C_CLK_LOW_TIME_2; SCL(1);
  I2C_Error = true;
  return false;
}
//-------------------------------------------------------------------------------------------------------------------------------
bool I2C_Stop()
{
  if(GetSCL() == 0)
	{
    SDA(0); I2C_CLK_LOW_TIME_2;
	}
  else
	{
    SCL(0); I2C_CLK_LOW_TIME_1; SDA(0); I2C_CLK_LOW_TIME_2;
	}
  SCL(1); I2C_STOP_TIME; SDA(1); I2C_STOP_TIME;
  if(GetSDA() == 0) goto labErr;
  return true;
labErr:
  I2C_Error=true;
  return false;
}
//-------------------------------------------------------------------------------------------------------------------------------
bool I2C_PutByte(Byte Smb)
{
  int i;
  bool rpar;

  if(GetSCL()) return false;
  if(I2C_Error) return false;
  for(i=0;i < 8;i++, Smb <<= 1)
	{
    if(Smb & 0x80) SDA(1);
    else SDA(0);
    I2C_CLK_LOW_TIME_2;
    SCL(1); I2C_CLK_HIGH_TIME; I2C_CLK_HIGH_TIME;
    SCL(0);
    I2C_CLK_LOW_TIME_1;
	}
  SDA(1);
  I2C_CLK_LOW_TIME_2;
  SCL(1); I2C_CLK_HIGH_TIME; I2C_CLK_HIGH_TIME;
  rpar = !GetSDA();
  SCL(0); I2C_CLK_LOW_TIME_1;
  return rpar;
}
//-------------------------------------------------------------------------------------------------------------------------------
bool I2C_GetByte(uint8* Smb,bool Ack)
{
  int i;
  uint8 vSmb=0;

  if(GetSCL()) return false;
  if(I2C_Error) return false;
  for(i=0;i < 8;i++)
	{
    vSmb<<=1;
    I2C_CLK_LOW_TIME_2; SCL(1); I2C_CLK_HIGH_TIME; I2C_CLK_HIGH_TIME;
    if(GetSDA()) vSmb|=0x1;
    SCL(0); I2C_CLK_LOW_TIME_1;
	}
  if(Ack) SDA(0);
  else SDA(1);
  I2C_CLK_LOW_TIME_2;
  if(!Ack && (GetSDA() == 0)) goto labErr;
  SCL(1); I2C_CLK_HIGH_TIME; I2C_CLK_HIGH_TIME;
  SCL(0); I2C_CLK_LOW_TIME_1;
  SDA(1);
  *Smb=vSmb;
  return true;
labErr:
  SCL(0); SDA(1);
  SCL(1);
  I2C_Error=true;
  return false;
}
//-------------------------------------------------------------------------------------------------------------------------------
bool I2C_TestLine()
{
  I2C_Init();
  if(I2C_Error) return false;
  return true;
}
//-------------------------------------------------------------------------------------------------------------------------------
